<?php
/**
 * Georgian language strings.
 *
 * This file is part of ADOdb, a Database Abstraction Layer library for PHP.
 *
 * @package ADOdb
 * @link https://adodb.org Project's web site and documentation
 * @link https://github.com/ADOdb/ADOdb Source code and issue tracker
 *
 * The ADOdb Library is dual-licensed, released under both the BSD 3-Clause
 * and the GNU Lesser General Public Licence (LGPL) v2.1 or, at your option,
 * any later version. This means you can use it in proprietary products.
 * See the LICENSE.md file distributed with this source code for details.
 * @license BSD-3-Clause
 * @license LGPL-2.1-or-later
 *
 * @copyright 2000-2013 John Lim
 * @copyright 2014 Damien Regad, Mark Newnham and the ADOdb community
 * @author Temuri Doghonadze <temuri.doghonadze@gmail.com>
 */

$ADODB_LANG_ARRAY = array (
             'LANG'                      => 'ka',
            DB_ERROR                    => 'უცნობი შეცდომა',
            DB_ERROR_ALREADY_EXISTS     => 'უკვე არსებობს',
            DB_ERROR_CANNOT_CREATE      => 'შექმნა შეუძლებელია',
            DB_ERROR_CANNOT_DELETE      => 'წაშლა შეუძლებელია',
            DB_ERROR_CANNOT_DROP        => 'წაშლა შეუძლებელია (drop)',
            DB_ERROR_CONSTRAINT         => 'შეზღუდვის პირობის დარღვევა',
            DB_ERROR_DIVZERO            => '0-ზე გაყოფა',
            DB_ERROR_INVALID            => 'არასწორია',
            DB_ERROR_INVALID_DATE       => 'არასწორი თარიღი ან დრო',
            DB_ERROR_INVALID_NUMBER     => 'არასწორი რიცხვი',
            DB_ERROR_MISMATCH           => 'შეცდომა',
            DB_ERROR_NODBSELECTED       => 'მონაცემთა ბაზა არჩეული არაა',
            DB_ERROR_NOSUCHFIELD        => 'ველი არ არსებობს',
            DB_ERROR_NOSUCHTABLE        => 'ცხრილი არ არსებობს',
            DB_ERROR_NOT_CAPABLE        => 'მონაცემთა ბაზა მზად არაა',
            DB_ERROR_NOT_FOUND          => 'აღმოჩენილი არაა',
            DB_ERROR_NOT_LOCKED         => 'დაბლოკილი არაა',
            DB_ERROR_SYNTAX             => 'სინტაქსური შეცდომა',
            DB_ERROR_UNSUPPORTED        => 'მხარდაჭერიი არაა',
            DB_ERROR_VALUE_COUNT_ON_ROW => 'მწკრივის მნიშვნელობების მთვლელი',
            DB_ERROR_INVALID_DSN        => 'არასწორი DSN',
            DB_ERROR_CONNECT_FAILED     => 'დაკავშირება შეუძლებელია',
            0                               => 'შეცდომის გარეშე', // DB_OK
            DB_ERROR_NEED_MORE_DATA     => 'მიწოდებულია არასაკმარისი მონაცემები',
            DB_ERROR_EXTENSION_NOT_FOUND=> 'გაფართოება აღმოჩენილი არაა',
            DB_ERROR_NOSUCHDB           => 'მონაცემთა ბაზა არ არსებობს',
            DB_ERROR_ACCESS_VIOLATION   => 'არასაკმარისი წვდომები'
);
